@@ -82,10 +82,11 @@ typedef struct
 	coeff_t const	*coeffs ;
 
 	int		b_current, b_end, b_real_end, b_len ;
-	float	*pdata ;
 	float	buffer [1] ;
 } SINC_FILTER ;
 
+static int sinc_process (SRC_PRIVATE *psrc, SRC_DATA *data) ;
+
 static double calc_output (SINC_FILTER *filter, increment_t increment, increment_t start_filter_index, int ch) ;
 
 static void prepare_data (SINC_FILTER *filter, SRC_DATA *data, int half_filter_chan_len) ;
@@ -147,7 +148,7 @@ sinc_get_description (int src_enum)
 int
 sinc_set_converter (SRC_PRIVATE *psrc, int src_enum)
 {	SINC_FILTER *filter, temp_filter ;
-	int count ;
+	int count, bits ;
 
 	/* Quick sanity check. */
 	if (SHIFT_BITS >= sizeof (increment_t) * 8 - 1)
@@ -172,26 +173,26 @@ sinc_set_converter (SRC_PRIVATE *psrc, int src_enum)
 	switch (src_enum)
 	{	case SRC_SINC_BEST_QUALITY :
 				temp_filter.coeffs = high_qual_coeffs ;
-				temp_filter.coeff_half_len = (sizeof (high_qual_coeffs) / sizeof (coeff_t)) - 1 ;
+				temp_filter.coeff_half_len = ARRAY_LEN (high_qual_coeffs) - 1 ;
 				temp_filter.index_inc = 128 ;
 				temp_filter.has_diffs = SRC_FALSE ;
-				temp_filter.coeff_len = sizeof (high_qual_coeffs) / sizeof (coeff_t) ;
+				temp_filter.coeff_len = ARRAY_LEN (high_qual_coeffs) ;
 				break ;
 
 		case SRC_SINC_MEDIUM_QUALITY :
 				temp_filter.coeffs = mid_qual_coeffs ;
-				temp_filter.coeff_half_len = (sizeof (mid_qual_coeffs) / sizeof (coeff_t)) - 1 ;
+				temp_filter.coeff_half_len = ARRAY_LEN (mid_qual_coeffs) - 1 ;
 				temp_filter.index_inc = 128 ;
 				temp_filter.has_diffs = SRC_FALSE ;
-				temp_filter.coeff_len = sizeof (mid_qual_coeffs) / sizeof (coeff_t) ;
+				temp_filter.coeff_len = ARRAY_LEN (mid_qual_coeffs) ;
 				break ;
 
 		case SRC_SINC_FASTEST :
 				temp_filter.coeffs = fastest_coeffs ;
-				temp_filter.coeff_half_len = (sizeof (fastest_coeffs) / sizeof (coeff_t)) - 1 ;
+				temp_filter.coeff_half_len = ARRAY_LEN (fastest_coeffs) - 1 ;
 				temp_filter.index_inc = 128 ;
 				temp_filter.has_diffs = SRC_FALSE ;
-				temp_filter.coeff_len = sizeof (fastest_coeffs) / sizeof (coeff_t) ;
+				temp_filter.coeff_len = ARRAY_LEN (fastest_coeffs) ;
 				break ;
 
 		default :
@@ -203,7 +204,7 @@ sinc_set_converter (SRC_PRIVATE *psrc, int src_enum)
 	** a better way. Need to look at prepare_data () at the same time.
 	*/
 
-	temp_filter.b_len = 1000 + 2 * lrint (ceil (temp_filter.coeff_len / (temp_filter.index_inc * 1.0) * SRC_MAX_RATIO)) ;
+	temp_filter.b_len = 1000 + 2 * lrint (0.5 + temp_filter.coeff_len / (temp_filter.index_inc * 1.0) * SRC_MAX_RATIO) ;
 	temp_filter.b_len *= temp_filter.channels ;
 
 	if ((filter = calloc (1, sizeof (SINC_FILTER) + sizeof (filter->buffer [0]) * (temp_filter.b_len + temp_filter.channels))) == NULL)
@@ -216,9 +217,11 @@ sinc_set_converter (SRC_PRIVATE *psrc, int src_enum)
 
 	sinc_reset (psrc) ;
 
-	count = (filter->coeff_half_len * INT_TO_FP (1)) / FP_ONE ;
+	count = filter->coeff_half_len ;
+	for (bits = 0 ; (1 << bits) < count ; bits++)
+		count |= (1 << bits) ;
 
-	if (abs (count - filter->coeff_half_len) >= 1)
+	if (bits + SHIFT_BITS - 1 >= (int) (sizeof (increment_t) * 8))
 		return SRC_ERR_FILTER_LEN ;
 
 	return SRC_ERR_NO_ERROR ;
@@ -247,10 +250,10 @@ sinc_reset (SRC_PRIVATE *psrc)
 **	Beware all ye who dare pass this point. There be dragons here.
 */
 
-int
+static int
 sinc_process (SRC_PRIVATE *psrc, SRC_DATA *data)
 {	SINC_FILTER *filter ;
-	double		input_index, src_ratio, count, float_increment, terminate ;
+	double		input_index, src_ratio, count, float_increment, terminate, rem ;
 	increment_t	increment, start_filter_index ;
 	int			half_filter_chan_len, samples_in_hand, ch ;
 
@@ -273,21 +276,16 @@ sinc_process (SRC_PRIVATE *psrc, SRC_DATA *data)
 	count = (filter->coeff_half_len + 2.0) / filter->index_inc ;
 	if (MIN (psrc->last_ratio, data->src_ratio) < 1.0)
 		count /= MIN (psrc->last_ratio, data->src_ratio) ;
-	count = lrint (ceil (count)) ;
 
 	/* Maximum coefficientson either side of center point. */
 	half_filter_chan_len = filter->channels * (lrint (count) + 1) ;
 
 	input_index = psrc->last_position ;
-	if (input_index >= 1.0)
-	{	filter->b_current = (filter->b_current + filter->channels * lrint (floor (input_index))) % filter->b_len ;
-		input_index -= floor (input_index) ;
-		} ;
-
 	float_increment = filter->index_inc ;
 
-	filter->b_current = (filter->b_current + filter->channels * lrint (floor (input_index))) % filter->b_len ;
-	input_index -= floor (input_index) ;
+	rem = fmod (input_index, 1.0) ;
+	filter->b_current = (filter->b_current + filter->channels * lrint (input_index - rem)) % filter->b_len ;
+	input_index = rem ;
 
 	terminate = 1.0 / src_ratio + 1e-20 ;
 
@@ -330,9 +328,10 @@ sinc_process (SRC_PRIVATE *psrc, SRC_DATA *data)
 
 		/* Figure out the next index. */
 		input_index += 1.0 / src_ratio ;
+		rem = fmod (input_index, 1.0) ;
 
-		filter->b_current = (filter->b_current + filter->channels * lrint (floor (input_index))) % filter->b_len ;
-		input_index -= floor (input_index) ;
+		filter->b_current = (filter->b_current + filter->channels * lrint (input_index - rem)) % filter->b_len ;
+		input_index = rem ;
 		} ;
 
 	psrc->last_position = input_index ;
